@@ -253,8 +253,9 @@ def build_particles(_context=None, _buffer=None, _offset=None, _capacity=None,
         assert at_element == expected_at_element or (
                 at_element < expected_at_element and
                       all([isinstance(tracker.line.element_dict[nn], xt.Drift)
+                           or tracker.line.element_dict[nn].__class__.__name__.startswith('Limit')
                 for nn in tracker.line.element_names[at_element:expected_at_element]])), (
-            "`match_at_s` can only be placed in the drifts upstream of the "
+            "`match_at_s` can only be placed in the drifts downstream of the "
             "specified `at_element`. No active element can be present in between."
             )
         (tracker_rmat, _
